@@ -2,7 +2,7 @@
  * Copyright (C) 2004 Lukas Lipka <lukas@pmad.net>
  *           (C) 2004 Michael Vogt <mvo@debian.org>
  *           (C) 2004 Michiel Sikkes <michiel@eyesopened.nl>
- *           (C) 2004-2006 Canonical
+ *           (C) 2004-2009 Canonical
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,6 @@
 #endif
 
 #include <signal.h>
-#include <gnome.h>
 #include <grp.h>
 #include <pwd.h>
 #include <limits.h>
@@ -33,9 +32,7 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-monitor.h>
+#include <gio/gio.h>
 
 #include "update-notifier.h"
 #include "hal.h"
@@ -66,8 +63,37 @@ static gboolean FORCE_START=FALSE;
 // force gksu for all menu actions
 static gboolean FORCE_GKSU=FALSE;
 
-// delay on startup
-static int STARTUP_DELAY=60000;
+// delay on startup (to make session startup faster)
+static int STARTUP_DELAY=5;
+
+// global debug options
+int HOOK_DEBUG = 0;
+int UPDATE_DEBUG = 0;
+int INOTIFY_DEBUG = 0;
+
+// logging stuff
+static void debug_log_handler (const gchar   *log_domain,
+                              GLogLevelFlags log_level,
+                              const gchar   *message,
+                              gpointer       user_data)
+{
+   if (HOOK_DEBUG > 0 && strcmp(log_domain, "hooks") == 0)
+       g_log_default_handler (log_domain, log_level, message, user_data);
+   if (UPDATE_DEBUG > 0 && strcmp(log_domain, "update") == 0)
+       g_log_default_handler (log_domain, log_level, message, user_data);
+   if (INOTIFY_DEBUG > 0 && strcmp(log_domain, "inotify") == 0)
+       g_log_default_handler (log_domain, log_level, message, user_data);
+
+}
+
+static inline void
+g_debug_inotify(const char *msg, ...)
+{
+   va_list va;
+   va_start(va, msg);
+   g_logv("inotify",G_LOG_LEVEL_DEBUG, msg, va);
+   va_end(va);
+}
 
 void invoke(gchar *cmd, gchar *desktop, gboolean with_gksu)
 {
@@ -120,18 +146,18 @@ trayapplet_create (TrayApplet *un, char *name)
   - /var/lib/apt/periodic/update-success-stamp
 */
 void
-monitor_cb(GnomeVFSMonitorHandle *handle,
-	   const gchar *monitor_uri,
-	   const gchar *info_uri,
-	   GnomeVFSMonitorEventType event_type,
+monitor_cb(GFileMonitor *handle,
+	   GFile *monitor_f,
+	   GFile *other_monitor_f,
+	   GFileMonitorEvent event_type,
 	   gpointer user_data)
 {
    UpgradeNotifier *un = (UpgradeNotifier*)user_data;
 
+   gchar *info_uri = g_file_get_path(monitor_f);
 #if 0
-   g_print("monitor_uri: %s\n", monitor_uri);
-   g_print("info_uri: %s\n", info_uri);
-   g_print("event_type: %i\n",event_type);
+   g_debug_inotify("info_uri: %s\n", info_uri);
+   g_debug_inotify("event_type: %i\n",event_type);
 #endif
 
    // we ignore lock file events because we can only get 
@@ -140,29 +166,31 @@ monitor_cb(GnomeVFSMonitorHandle *handle,
       return;
 
    // look for apt-get install/update
-   if(g_str_has_prefix(info_uri,"file:///var/lib/apt/") 
-      || g_str_has_prefix(info_uri,"file:///var/cache/apt/")
-      || strcmp(info_uri,"file:///var/lib/dpkg/status") == 0) {
+   if(g_str_has_prefix(info_uri,"/var/lib/apt/") 
+      || g_str_has_prefix(info_uri,"/var/cache/apt/")
+      || strcmp(info_uri,"/var/lib/dpkg/status") == 0) {
+      g_debug_inotify("apt_get_running=TRUE");
       un->apt_get_runing=TRUE;
    } 
    if(strstr(info_uri, "/var/lib/update-notifier/dpkg-run-stamp") ||
       strstr(info_uri, "/var/lib/apt/periodic/update-success-stamp")) {
+      g_debug_inotify("dpkg_was_run=TRUE");
       un->dpkg_was_run = TRUE;
    } 
    if(strstr(info_uri, REBOOT_FILE)) {
-      //g_print("reboot required\n");
+      g_debug_inotify("reboot required\n");
       un->reboot_pending = TRUE;
    }
    if(strstr(info_uri, HOOKS_DIR)) {
-      //g_print("new hook!\n");
+      g_debug_inotify("new hook!\n");
       un->hook_pending = TRUE;
    }
    if(strstr(info_uri, CRASHREPORT_DIR)) {
-      //g_print("crashreport found\n");
+      g_debug_inotify("crashreport found\n");
       un->crashreport_pending = TRUE;
    }
    if(strstr(info_uri, UNICAST_LOCAL_AVAHI_FILE)) {
-      //g_print("avahi disabled due to unicast .local domain\n");
+      g_debug_inotify("avahi disabled due to unicast .local domain\n");
       un->unicast_local_avahi_pending = TRUE;
    }
 }
@@ -280,8 +308,7 @@ void u_abort(gchar *msg)
 gboolean monitor_init(UpgradeNotifier *un)
 {
    int i;
-   GnomeVFSResult res;
-   GnomeVFSMonitorHandle *monitor_handle;
+   GFileMonitor *monitor_handle;
 
    // monitor thise dirs
    static const char *monitor_dirs[] = { 
@@ -291,10 +318,13 @@ gboolean monitor_init(UpgradeNotifier *un)
       CRASHREPORT_DIR,
       NULL};
    for(i=0;monitor_dirs[i] != NULL;i++) {
-      res = gnome_vfs_monitor_add(&monitor_handle,
-				  monitor_dirs[i],
-				  GNOME_VFS_MONITOR_DIRECTORY,
-				  monitor_cb, un);
+      GError err;
+      GFile *gf = g_file_new_for_path(monitor_dirs[i]);
+      monitor_handle = g_file_monitor_directory(gf, 0, NULL, &err);
+      if(monitor_handle)
+	 g_signal_connect(monitor_handle, "changed", monitor_cb, un);
+      else
+	 g_warning("can not add '%s'\n", monitor_dirs[i]);
    }
 
    // and those files
@@ -306,10 +336,13 @@ gboolean monitor_init(UpgradeNotifier *un)
       UNICAST_LOCAL_AVAHI_FILE,
       NULL};
    for(i=0;monitor_files[i] != NULL;i++) {
-      res = gnome_vfs_monitor_add(&monitor_handle,
-				  monitor_files[i],
-				  GNOME_VFS_MONITOR_FILE,
-				  monitor_cb, un);
+      GError err;
+      GFile *gf = g_file_new_for_path(monitor_files[i]);
+      monitor_handle = g_file_monitor_file(gf, 0, NULL, &err);
+      if(monitor_handle)
+	 g_signal_connect(monitor_handle, "changed", monitor_cb, un);
+      else
+	 g_warning("can not add '%s'\n", monitor_dirs[i]);
    }
 
    g_timeout_add (TIMEOUT_FAM, (GSourceFunc)file_monitor_periodic_check, un);
@@ -324,13 +357,13 @@ gboolean monitor_init(UpgradeNotifier *un)
 static gboolean
 tray_icons_init(UpgradeNotifier *un)
 {
-   //g_debug("tray_icons_init");
+   //g_debug_inotify("tray_icons_init");
 
    /* new upates tray icon */
    un->update = g_new0 (TrayApplet, 1);
 
    // check if the updates icon should be displayed
-   if (in_admin_group()) {
+   if (in_admin_group() || FORCE_START) {
       trayapplet_create(un->update, "software-update-available");
       update_tray_icon_init(un->update);
    } else 
@@ -360,8 +393,16 @@ system_user()
    /* do not start for system users, e.g. the guest session 
     * (see /usr/share/gdm/guest-session/guest-session-setup.sh)
     */
+   int end_system_uid = 500;
+   GConfClient *gconf = gconf_client_get_default();
+   if (gconf) {
+      int i = gconf_client_get_int(gconf, GCONF_KEY_END_SYSTEM_UIDS, NULL);
+      if (i>0)
+	 end_system_uid = i;
+   }
+
    uid_t uid = getuid();
-   if (uid < 500)
+   if (uid < end_system_uid)
       return TRUE;
    return FALSE;
 }
@@ -404,9 +445,11 @@ in_admin_group()
 static GOptionEntry entries[] = 
 {
    { "debug-hooks", 0, 0, G_OPTION_ARG_NONE, &HOOK_DEBUG, "Enable hooks debugging"},
+   { "debug-updates", 0, 0, G_OPTION_ARG_NONE, &UPDATE_DEBUG, "Enable updates/autolaunch debugging"},
+   { "debug-inotify", 0, 0, G_OPTION_ARG_NONE, &INOTIFY_DEBUG, "Enable inotify debugging"},
    { "force", 0, 0, G_OPTION_ARG_NONE, &FORCE_START, "Force start even if the user is not in the admin group"},
    { "force-use-gksu", 0, 0, G_OPTION_ARG_NONE, &FORCE_GKSU, "Force running all commands (update-manager, synaptic) with gksu" },
-   { "startup-delay", 0, 0, G_OPTION_ARG_INT, &STARTUP_DELAY, "Delay statup by given amount of miliseconds" },
+   { "startup-delay", 0, 0, G_OPTION_ARG_INT, &STARTUP_DELAY, "Delay statup by given amount of seconds" },
    { NULL }
 };
 
@@ -425,35 +468,37 @@ main (int argc, char *argv[])
 			    argc, argv, 
 			    GNOME_PARAM_GOPTION_CONTEXT, context,
 			    GNOME_PARAM_NONE);
+
 	notify_init("update-notifier");
         bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
         bind_textdomain_codeset(PACKAGE, "UTF-8");
         textdomain(PACKAGE);
 
+	// setup a custom debug log handler
+	g_log_set_handler ("inotify", G_LOG_LEVEL_DEBUG,
+			   debug_log_handler, NULL);
+	g_log_set_handler ("hooks", G_LOG_LEVEL_DEBUG,
+			   debug_log_handler, NULL);
+	g_log_set_handler ("update", G_LOG_LEVEL_DEBUG,
+			   debug_log_handler, NULL);
+
 	g_set_application_name (_("update-notifier"));
 	gtk_window_set_default_icon_name ("update-notifier");
 
 	//g_print("starting update-notifier\n");
 
 	// do not run as system user (e.g. guest user)
-	if (system_user()) {
+	if (system_user() && !FORCE_START) {
 	   g_warning("not starting for system user\n");
 	   exit(0);
 	}
-	
-	client = gnome_master_client ();
-	if (up_get_clipboard ())
-	  gnome_client_set_restart_style (client, GNOME_RESTART_ANYWAY);
-	else {
-	   gnome_client_set_restart_style (client, GNOME_RESTART_NEVER);
+
+	// check if it is running already
+	if (!up_get_clipboard ()) {
 	   g_warning ("already running?\n");
 	   return 1;
 	}
 
-	/* Make sure we die when the session dies */
-	g_signal_connect (G_OBJECT (client), "die",
-			  G_CALLBACK (gtk_main_quit), NULL);
-	
 	/* Create the UpgradeNotifier object */
 	un = g_new0 (UpgradeNotifier, 1);
 
@@ -466,7 +511,7 @@ main (int argc, char *argv[])
 
 	// delay icon creation for 30s so that the desktop 
 	// login is not slowed down by update-notifier
-	g_timeout_add(STARTUP_DELAY, (GSourceFunc)(tray_icons_init), un);
+	g_timeout_add_seconds(STARTUP_DELAY, (GSourceFunc)(tray_icons_init), un);
 
         // initial check for avahi
         avahi_disabled_check();
